package ever.simpleCooking;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Stack;

import ever.pipeline.AbstractLocalInputAnalyzer;
import ever.pipeline.Case;
import ever.workflowRepresentation.Product;
import ever.workflowRepresentation.Sequence;
import ever.workflowRepresentation.SimpleProduct;
import ever.workflowRepresentation.Task;
import ever.workflowRepresentation.Workflow;
import ever.workflowRepresentation.WorkflowElement;
/**
 * This class searches for products with an indirect article. Observation showed, that often products where created which are tools. These products often contain
 * an indirect article.
 * @author Pol Schumacher, Wirtschaftsinformatik, Institut fuer Informatik, Goethe Universitaet Frankfurt
 *
 */
public class IndirectArticleCleaner extends AbstractLocalInputAnalyzer {

	private Workflow wf;
	private HashSet<String> cookware=null;
	@Override
	public Case analyze(Case c) {

		loadHashSet();
		wf = c.getWf();

		Sequence seq = wf.getTopSequence();		
		int size = seq.getAllElements().size();
		for (int i = 0; i < size; i++) {
			
			Task currentTask = (Task)seq.getAllElements().get(i);			
			ArrayList<Product> stack = currentTask.getAllInProducts();
			HashSet<Product> toRemove = new HashSet<Product>();
			for(Product p :stack)
			{
				String name = p.getName().trim().toLowerCase();
				
				if(isCookingTool(name))
				{
					toRemove.add(p);
					
				}
				
				if(name.matches("(^an|^a)( )(.)+"))
				{
					if(!isInProductList(name))
					{						
						toRemove.add(p);						
					}										
				}			
				if(name.equals("them")|name.contains("for ")|name.equals("well")|name.equalsIgnoreCase("it")|name.equalsIgnoreCase("degf")|name.contains("oven"))
					toRemove.add(p);
			}			
			
			for(Product p : toRemove)
				currentTask.removeInProduct(p);
						
			seq.putElementAt(currentTask, i);			
			
		}		
		wf.setTopSequence(seq);
		c.setWf(wf);		
		return c;
	}

	@Override
	public HashMap<Integer, Case> analyze(HashMap<Integer, Case> c) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void analyze() {
		// TODO Auto-generated method stub

	}
	
	private boolean isInProductList(String name)
	{
		if(name.contains("water")|name.contains("slice")|name.contains("tablespoon")|name.contains("layer")|name.contains("piece"))
		{
//			System.out.println(name);
			return true;
		}
		String[] ns = name.split(" ");
		
		for(Product p : wf.all)
		{
			boolean firstMatch=false;
			String[] split=p.getName().split(" ");
			for(String s1:ns)
			{
				if(s1.equals("a")|s1.equals("an"))
					continue;
				
				
				for(String s2: split)
				{
					if(s2.equals(s1))
					{
						
						if(firstMatch)
						{
//							System.out.println(name +"===="+p.getName());
							return true;
						}
						
						firstMatch=true;
					}
				}
			}
			if((ns.length==1|split.length==1)&firstMatch)
				return true;
		}
//		System.out.println("========================================================================================="+name);
		return false;
	}
	
	private void loadHashSet()
	{
		cookware=new HashSet<String>();
		BufferedReader reader = null;
		try {
			reader = new BufferedReader(new FileReader(new File("cookware.txt")));
		} catch (FileNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		String line = null;
		try {
			line = reader.readLine();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		while(line!=null)
		{
			if(line.trim().length()>0)
			cookware.add(line.toLowerCase().trim());
			try {
				line=reader.readLine();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
			
	}
	
	private boolean isCookingTool(String name)
	{
		for(String s:cookware)
		{
			if(name.contains(s))
			{
//				System.out.println(name + " is cookware --->"+s);
				return true;
			}
		}
		return false;
	}

}
