package ever.simpleCooking;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import javax.xml.transform.stream.StreamSource;

import net.sf.saxon.s9api.DocumentBuilder;
import net.sf.saxon.s9api.Processor;
import net.sf.saxon.s9api.QName;
import net.sf.saxon.s9api.SaxonApiException;
import net.sf.saxon.s9api.Serializer;
import net.sf.saxon.s9api.XQueryCompiler;
import net.sf.saxon.s9api.XQueryEvaluator;
import net.sf.saxon.s9api.XQueryExecutable;
import net.sf.saxon.s9api.XdmAtomicValue;
import net.sf.saxon.s9api.XdmNode;
import ever.pipeline.AbstractLocalInputAnalyzer;
import ever.pipeline.Case;
import ever.pipeline.CaseFrame;
import ever.pipeline.Clause;
import ever.pipeline.Sentence;
import ever.pipeline.XmlHandler;
import ever.workflowRepresentation.And;
import ever.workflowRepresentation.EmptyProductException;
import ever.workflowRepresentation.NotAProductException;
import ever.workflowRepresentation.Sequence;
import ever.workflowRepresentation.SimpleFacet;
import ever.workflowRepresentation.SimpleProduct;
import ever.workflowRepresentation.Task;
import ever.workflowRepresentation.Workflow;
import ever.workflowRepresentation.WorkflowElement;

/**
 * This class applies several pattern to extract a parallel control-flow
 * @author Pol Schumacher, Wirtschaftsinformatik, Institut fuer Informatik, Goethe Universitaet Frankfurt Input workflow must be flat
 */
public class LocalAndDetector extends AbstractLocalInputAnalyzer {

	private String raw;
	private ArrayList<CaseFrame> caseFrameList;
	private ArrayList<Clause> clauseList;
	private Sentence s;
	private Case cas;

	@Override
	public Case analyze(Case cas) {
		this.cas = cas;
		Workflow wf = cas.getWf();
		Sequence seq = null;
		ArrayList<WorkflowElement> taskList = new ArrayList<WorkflowElement>();
		if (wf == null) {
			wf = new Workflow();
			seq = new Sequence(null,null);

		} else {
			seq = (Sequence) wf.getTopSequence();
		}
		ArrayList<Sentence> senList = cas.getSentences();
		And a = null;
		for (int i = 0; i < senList.size(); i++) {
			caseFrameList = senList.get(i).getCaseFrames();
			clauseList = senList.get(i).getClauseList();
			raw = senList.get(i).getRaw();
			s = senList.get(i);
			if (checkWhilePresCont()) {
				a = getWhilePresCont();

			} else { 
				if (checkFrequentlyCase()) {
					a = (And) getFrequentlyAnd();
				} else {
					if (checkReservingCase()) {
						a = (And) getReservingAnd();
					} else {
						if (checkStirCase()) {
							a = (And) getStirAnd();
						} else {
							if (checkMeanwhile()) {
								a = (And) getMeanwhileAnd();
							} else {
								if(checkTurningCase())
									a=getTurningAnd();
								else									
									continue;
							}
						}
					}
				}
			}
			// Put new task on the right position in sequence			
			int pos = super.getNextRelated(senList, s, seq);					

			seq.putElementAt(a, pos);

		}

		wf.setTopSequence(seq);
		cas.setWf(wf);
		return cas;
	}

	private String getMeanwhileStepString(String raw) {
		String query = "declare variable $sentence as xs:string external; \n"
				+ "let $steps := /howto/steps/step \n"
				+ "for $i at $index in $steps \n"
				+ "return if(starts-with($i/lower-case(text()),$sentence)) \n"
				+ "then $i \n" + "else \"\"";

		Processor proc = new Processor(false);
		DocumentBuilder newDocumentBuilder = proc.newDocumentBuilder();

		XQueryCompiler newXQueryCompiler = proc.newXQueryCompiler();
		XQueryExecutable compile = null;
		try {
			XdmNode input = newDocumentBuilder.build(new StreamSource(
					new ByteArrayInputStream(cas.getXmlString().getBytes())));
			compile = newXQueryCompiler.compile(query);

			XQueryEvaluator load = compile.load();

			load.setContextItem(input);
			load.setExternalVariable(new QName("sentence"), new XdmAtomicValue(
					raw));
			Serializer serial = new Serializer();
			ByteArrayOutputStream outs = new ByteArrayOutputStream();
			serial.setOutputStream(outs);
			serial.setOutputProperty(Serializer.Property.METHOD, "text");
			serial.setOutputProperty(Serializer.Property.OMIT_XML_DECLARATION,
					"yes");
			serial.setOutputProperty(Serializer.Property.ENCODING, "utf-8");
			load.run(serial);
			return outs.toString().trim();

		} catch (SaxonApiException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

	private String getPreMeanwhileStepString(String raw) {
		String query = "declare variable $sentence as xs:string external; \n"
				+ "let $steps := /howto/steps/step \n"
				+ "for $i at $index in $steps \n"
				+ "return if(starts-with($i/following::step[position()=1]/lower-case(text()),$sentence)) \n"
				+ "then $i \n" + "else \"\"";

		Processor proc = new Processor(false);
		DocumentBuilder newDocumentBuilder = proc.newDocumentBuilder();

		XQueryCompiler newXQueryCompiler = proc.newXQueryCompiler();
		XQueryExecutable compile = null;
		try {
			XdmNode input = newDocumentBuilder.build(new StreamSource(
					new ByteArrayInputStream(cas.getXmlString().getBytes())));
			compile = newXQueryCompiler.compile(query);

			XQueryEvaluator load = compile.load();

			load.setContextItem(input);
			load.setExternalVariable(new QName("sentence"), new XdmAtomicValue(
					raw));
			Serializer serial = new Serializer();
			ByteArrayOutputStream outs = new ByteArrayOutputStream();
			serial.setOutputStream(outs);
			serial.setOutputProperty(Serializer.Property.METHOD, "text");
			serial.setOutputProperty(Serializer.Property.OMIT_XML_DECLARATION,
					"yes");
			serial.setOutputProperty(Serializer.Property.ENCODING, "utf-8");
			load.run(serial);
			return outs.toString().trim();

		} catch (SaxonApiException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return "";
		}
	}

	private boolean checkMeanwhile() {
		String query = "declare variable $sentence as xs:string external; \n"
				+ "let $steps := /howto/steps/step \n"
				+ "for $i at $index in $steps \n"
				+ "return if(starts-with($i,\"Meanwhile\") and starts-with(lower-case($i),$sentence) and count($steps)>1) \n"
				+ "then \"true\" \n" + "else \"\"";

		Processor proc = new Processor(false);
		DocumentBuilder newDocumentBuilder = proc.newDocumentBuilder();

		XQueryCompiler newXQueryCompiler = proc.newXQueryCompiler();
		XQueryExecutable compile = null;

		try {
			XdmNode input = newDocumentBuilder.build(new StreamSource(
					new ByteArrayInputStream(cas.getXmlString().getBytes())));
			compile = newXQueryCompiler.compile(query);

			XQueryEvaluator load = compile.load();

			load.setContextItem(input);
			load.setExternalVariable(new QName("sentence"), new XdmAtomicValue(
					raw));
			Serializer serial = new Serializer();
			ByteArrayOutputStream outs = new ByteArrayOutputStream();
			serial.setOutputStream(outs);
			serial.setOutputProperty(Serializer.Property.METHOD, "text");
			serial.setOutputProperty(Serializer.Property.OMIT_XML_DECLARATION,
					"yes");
			serial.setOutputProperty(Serializer.Property.ENCODING, "utf-8");
			load.run(serial);
			boolean ret = false;
			try {
				ret = Boolean.parseBoolean(outs.toString().trim());
			} catch (NumberFormatException ex) {
				System.out.println("stop");
			}

			if (ret) {
				if (cas.getSentences().indexOf(s) > 0)
					return true;

			} else
				return false;

			// System.out.println(ret);
			return ret;
		} catch (SaxonApiException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

	}

	// private boolean checkPreMeanwhile()
	// {
	// int index=XmlHandler.isPreMeanwhileStep(raw);
	// if(index>-1)
	// {
	// andType=AndLocator.PRE_MEANWHILE;
	// return true;
	// }
	// return false;
	// }
	/*
	 * Should locate sentence with While present continious e.g.: While potatoes
	 * are boiling, place the cabbage into a pot with several tablespoons of
	 * water;
	 */
	private boolean checkWhilePresCont() {

		if (raw.startsWith("while")) {
			boolean aux_verb = false;
			for (Clause c : clauseList) {
				if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):")
						& c.getInfos().get(0).contains("(root: be")) {
					aux_verb = true;
				}

				if (aux_verb & c.getWholeTyp().contains("NP SEGMENT")
						& c.getWholeTyp().contains("(DOBJ")
						& c.getValues().get(0).endsWith("ing")) {

					return true;
				}

				if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):")
						& c.getInfos().get(0).contains("(root: be")) {
					aux_verb = true;
					if (c.getInfos().size() > 1) {
						if (c.getInfos().get(1).contains("(MOR)(V PARTICIPLE)")) {

							return true;
						}
					}
				}

				if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):")
						& c.getInfos().get(0).contains("(root: be")) {
					aux_verb = true;
					if (c.getInfos().size() > 1) {
						if (c.getInfos().get(1).contains("(MOR)(V PARTICIPLE)")) {

							return true;
						}
					}
				}

			}

		}
		return false;
	}

	/*
	 * Should locate And flows for cases like :"Drain peaches and apricots >,
	 * reserving juice.
	 */
	private boolean checkReservingCase() {
		if (raw.contains(" reserving")) {
			// andType=RESERVING_TYPE;

			return true;
		}

		return false;
	}

	private And getTurningAnd() {
		And a = new And(cas.getWf().getTopSequence(),s);
		
		// other path		
		ArrayList<WorkflowElement> tl = s.getRelatedElem();
		Sequence seq = new Sequence(a,s);
		for(WorkflowElement wfe : tl)
		{
			seq.addWfElement(wfe);
			cas.getWf().getTopSequence().removeWfe(wfe);
		}
					
		a.addOption(seq);
					
					
		if (raw.contains("turning occasionally")) {
			Task t = new Task(a,s,"turn");
			SimpleFacet sourceSentence=new SimpleFacet("SourceSentence", raw);
			t.addFacet(sourceSentence);
			SimpleFacet f = new SimpleFacet("verb modificator", "occasionally");
			t.addFacet(f);
			seq = new Sequence(a,s);
			seq.addWfElement(t);			
			a.addOption(seq);
		}

		

//		Sequence seq = new Sequence(a);
//		boolean containsCook = false;
//		for (Task t : tl) {
//			if (t.getName().contains("cook"))
//				containsCook = true;
//		}
//		for (Task t : tl) {
//			if (containsCook) {
//				if (!t.getName().contains("cook")) {
//					wf.addWorkflowElement(t);
//				} else {
//					seq.addWfElement(t);
//				}
//			} else {
//				seq.addWfElement(t);
//			}
//		}
//		a.addOption(seq);
		s.addRelWfElement(a);
		a.addRelSentence(s);
		return a;
	}

	private boolean checkTurningCase() {
		if (raw.contains("turning occasionally")) {
			// andType=AndLocator.TURNING_TYPE;
			return true;
		}
		return false;
	}

	private boolean checkFrequentlyCase() {
		boolean stir = false, freq = false;

		if (raw.contains("frequently")) {
			for (Clause c : clauseList) {
				if (c.getValue().contains("stirring"))
					stir = true;

				if (c.getValue().contains("frequently"))
					freq = true;

			}
			if (stir & freq) {
				// andType=FREQUENTLY_TYPE;
				return true;
			}
		}
		return false;
	}

	// Task stir in combination with an adverb
	private boolean checkStirCase() {
		boolean stir = false, adverb = false;
		for (Clause c : clauseList) {
			if (c.getValue().contains("stirring")
					| c.getValue().contains("stir"))
				stir = true;

			if ((c.getValue().contains("reserving")
					| c.getValue().contains("occasionally")
					| c.getValue().contains("constantly")
					| c.getValue().contains("frequently") | c.getValue()
					.contains("thoroughly")) & stir)
				adverb = true;

		}
		if (stir & adverb) {
			return true;
		} else {
			return false;
		}

	}

	private And getWhilePresCont() {

		And a = new And(cas.getWf().getTopSequence(),s);
		boolean aux_verb = false;
		String tName = null;
		String pName = null;
		for (Clause c : clauseList) {
			if (c.getWholeTyp().contains("NP SEGMENT") & !aux_verb) {
				pName = c.getValue();
				pName = pName.replace("While", "");
				pName = pName.trim();
			}
			if (c.getWholeTyp().contains("VP SEGMENT (ACTIVE_VERB):")
					& c.getInfos().get(0).contains("(root: be")) {
				aux_verb = true;
				if (c.getInfos().size() > 1) {
					if (c.getInfos().get(1).contains("(MOR)(V PARTICIPLE)")) {
						tName = c.getValues().get(1);

						break;
					}
				}
			}
			if (aux_verb & c.getWholeTyp().contains("NP SEGMENT")
					& c.getWholeTyp().contains("(DOBJ")
					& c.getValues().get(0).endsWith("ing")) {
				tName = c.getValues().get(0);
				break;
			}
		}
		Task t = new Task(a,s, tName);
		SimpleFacet sourceSentence = new SimpleFacet("SourceSentence", raw);
		t.addFacet(sourceSentence);
		String[] ing = null;
		boolean empty = false;
		try {
			ing = pName.split("(>comma)|(>COMMA)|( and )|( or )");
			;
		} catch (NullPointerException npe) {
			empty = true;
		}
		if (!empty) {
			// Dirty fix to remove duplicate ingredients
			HashSet<String> tmp = new HashSet<String>();

			for (int i = 0; i < ing.length; i++) {
				// To prevent empty ingredients from being added to wf
				if (ing[i].isEmpty())
					continue;
				tmp.add(ing[i]);
			}
			for (String in : tmp) {
				// Add Product to df chain
				SimpleProduct sp = null;
				try {
					sp = new SimpleProduct(in, t.getId());
					t.addInProduct(sp);
					// df.addItem(sp, t.getId());

				} catch (EmptyProductException e) {
					continue;
				} catch (NotAProductException e) {
					continue;
				}
			}
		}
		Sequence seq = new Sequence(a,s);
		seq.addWfElement(t);
		a.addOption(seq);

		ArrayList<WorkflowElement> tList = null;
		tList = s.getRelatedElem();

		seq = new Sequence(a,s);
		for (WorkflowElement ta : tList) {
			ta.addRelSentence(s);
			seq.addWfElement(ta);
		}
		a.addOption(seq);

		return a;
	}

	public WorkflowElement getFrequentlyAnd() {
		And a = new And(cas.getWf().getTopSequence(),s);
		String lastInf = "";
		// Stirring path
		for (Clause cf : clauseList) {
			if (cf.getWholeTyp().contains("NP SEGMENT")) {
				for (String s : cf.getInfos()) {
					if (s.contains("(INF-MOR)(GER)"))
						lastInf = cf.getValues().get(cf.getInfos().indexOf(s));
				}
			}
			if (cf.getValue().contains("frequently") & !lastInf.isEmpty()) {
				Sequence seq = new Sequence(a,s);
				Task t = new Task();
				// Special case handling because its frequent
				if (lastInf.equalsIgnoreCase("stirring"))
					lastInf = "stir";

				SimpleFacet sourceSentence = new SimpleFacet("SourceSentence",
						raw);
				t.addFacet(sourceSentence);
				t.setName(lastInf);
				t.addFacet(new SimpleFacet("verb modificator", "frequently"));
				seq.addWfElement(t);
				a.addOption(seq);
			}
		}
		// other path
		Sequence seq = new Sequence(a,s);
		ArrayList<WorkflowElement> tl = s.getRelatedElem();
		for (WorkflowElement ta : tl) {
			ta.addRelSentence(s);
			seq.addWfElement(ta);
		}
		a.addOption(seq);
		a.addRelSentence(s);
		return a;
	}

	/*
	 * Should extract And flows for cases like :"Drain peaches and apricots >,
	 * reserving juice.
	 */
	private WorkflowElement getReservingAnd() {
		And a = new And(cas.getWf().getTopSequence(),s);
		// Case Frames containing reserving
		ArrayList<CaseFrame> reserveList = new ArrayList<CaseFrame>();

		// CaseFrames without reserving
		ArrayList<CaseFrame> mainList = new ArrayList<CaseFrame>();

		for (CaseFrame cf : caseFrameList) {
			if (cf.getTrigger().equals("RESERVING")) {
				reserveList.add(cf);
			} else {
				mainList.add(cf);
			}
		}

		// Clauses containing reserving
		ArrayList<Clause> reserveClauseList = new ArrayList<Clause>();

		// Clauses without reserving
		ArrayList<Clause> mainClauseList = new ArrayList<Clause>();
		for (Clause cl : clauseList) {
			if (cl.getValue().toLowerCase().contains("reserving")) {
				reserveClauseList.add(cl);
			} else {
				mainClauseList.add(cl);
			}
		}
		// Reserving path

		ArrayList<WorkflowElement> tl = new ArrayList<WorkflowElement>();
		// No task was extracted but it is there use second fallback
		if (tl.size() == 0) {
			boolean fres = false;
			Task t = new Task(a, s,"reserve");
			String product = "";
			for (Clause c : reserveClauseList) {
				for (String s : c.getValues()) {
					if (fres)
						product += s;

					if (s.toLowerCase().contains("reserving")) {
						fres = true;
					}
				}
			}
			try {
				t.addInProduct(new SimpleProduct(product, t.getId()));
			} catch (EmptyProductException e) {
				// just skip
			} catch (NotAProductException e) {
				// just skip
			}
			tl.add(t);
		}

		Sequence seq = new Sequence(a,s);
		for (WorkflowElement t : tl) {
			seq.addWfElement(t);
		}
		a.addOption(seq);

		// Other path
		seq = new Sequence(a,s);
		tl = s.getRelatedElem();
		for (WorkflowElement t : tl) {
			if (((Task) t).getName().equals("reserving"))
				continue;

			seq.addWfElement(t);
			t.addRelSentence(s);
		}
		a.addOption(seq);
		a.addRelSentence(s);
		return a;
	}

	private WorkflowElement getStirAnd() {
		And a = new And(cas.getWf().getTopSequence(),s);
		if (raw.contains("stirring constantly")) {
			Task t = new Task(a,s, "stir");
			SimpleFacet sourceSentence = new SimpleFacet("SourceSentence", raw);
			t.addFacet(sourceSentence);
			SimpleFacet f = new SimpleFacet("verb modificator", "constantly");
			t.addFacet(f);
			Sequence seq = new Sequence(a,s);
			seq.addWfElement(t);
			a.addOption(seq);
		}

		if (raw.contains("stirring occasionally")) {
			Task t = new Task(a,s, "stir");
			SimpleFacet sourceSentence = new SimpleFacet("SourceSentence", raw);
			t.addFacet(sourceSentence);
			SimpleFacet f = new SimpleFacet("verb modificator", "occasionally");
			t.addFacet(f);
			Sequence seq = new Sequence(a,s);
			seq.addWfElement(t);
			a.addOption(seq);
		}

		if (raw.contains("stirring thoroughly")) {
			Task t = new Task(a,s, "stir");
			SimpleFacet sourceSentence = new SimpleFacet("SourceSentence", raw);
			t.addFacet(sourceSentence);
			SimpleFacet f = new SimpleFacet("verb modificator", "thoroughly");
			t.addFacet(f);
			Sequence seq = new Sequence(a,s);
			seq.addWfElement(t);
			a.addOption(seq);
		}

		if (raw.contains("stirring frequently")) {
			Task t = new Task(a, s,"stir");
			SimpleFacet sourceSentence = new SimpleFacet("SourceSentence", raw);
			t.addFacet(sourceSentence);
			SimpleFacet f = new SimpleFacet("verb modificator", "frequently");
			t.addFacet(f);
			Sequence seq = new Sequence(a,s);
			seq.addWfElement(t);
			a.addOption(seq);
		}

		// other path

		ArrayList<WorkflowElement> tl = s.getRelatedElem();

		Sequence seq = new Sequence(a,s);
		boolean containsCook = false;
		for (WorkflowElement t : tl) {
			if (((Task) t).getName().contains("cook"))
				containsCook = true;
		}
		for (WorkflowElement t : tl) {
			if (containsCook) {
				if (!((Task) t).getName().contains("cook")) {
					// wf.addWorkflowElement(t);
				} else {
					seq.addWfElement(t);
				}
			} else {
				seq.addWfElement(t);
			}
		}
		a.addOption(seq);

		return a;
	}

	private WorkflowElement getMeanwhileAnd() {		
		And and = new And(cas.getWf().getTopSequence(),s);
		ArrayList<Sentence> meanStepSentences = new ArrayList<Sentence>();
		int currentIndex = cas.getSentences().indexOf(s);
		String meanwhileString = getMeanwhileStepString(s.getRaw())
				.toLowerCase();
		int shift = 1;
		Sentence sb = cas.getSentences().get(
				cas.getSentences().indexOf(s) + shift);
		meanStepSentences.add(s);
		while (meanwhileString.contains(sb.getRaw())) {
			meanStepSentences.add(sb);
			shift++;
			if(currentIndex+shift==cas.getSentences().size())
				break;
			
			try {
				sb = cas.getSentences().get(currentIndex + shift);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		Sequence seq = new Sequence(and,s);
		for (Sentence cs : meanStepSentences) {
			for (int i = 0; i < cs.getRelatedElem().size(); i++) {
				seq.addWfElement(cs.getRelatedElem().get(i));
				cas.getWf().getTopSequence()
						.removeWfe(cs.getRelatedElem().get(i));
				cs.removeRelWfElement(cs.getRelatedElem().get(i));

			}
			cs.addRelWfElement(and);
			and.addRelSentence(cs);
		}
		and.addOption(seq);
		s.addRelWfElement(and);

		String before = getPreMeanwhileStepString(raw).toLowerCase();
		// Collect all sentences which are belonging to the step before
		// meanwhile
		ArrayList<Sentence> preStepSentences = new ArrayList<Sentence>();

		shift = 1;
		if(cas.getSentences().indexOf(s)==0)
			sb=cas.getSentences().get(0);
		else
			sb = cas.getSentences().get(cas.getSentences().indexOf(s) - shift);
		
		while (before.contains(sb.getRaw())) {
			preStepSentences.add(0, sb);		
			
			boolean test = before.startsWith(sb.getRaw());
			if(before.equals(sb.getRaw())|test)
				break;
			
			shift++;
			try {
				sb = cas.getSentences().get(currentIndex - shift);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		seq = new Sequence(and,s);
		for (Sentence cs : preStepSentences) {
			for (int i = 0; i  < cs.getRelatedElem().size(); i = i) {
				seq.addWfElement(cs.getRelatedElem().get(i));
				cas.getWf().getTopSequence()
						.removeWfe(cs.getRelatedElem().get(i));
				cs.removeRelWfElement(cs.getRelatedElem().get(i));
			}
			cs.addRelWfElement(and);
			and.addRelSentence(cs);
		}
		and.addOption(seq);

		return and;
	}

	@Override
	public HashMap<Integer, Case> analyze(HashMap<Integer, Case> c) {
		// TODO Auto-generated method stub
		// throw new NotImplementedException();
		return null;

	}

	@Override
	public void analyze() {
		// throw new NotImplementedException();

	}

}
